@@ -28,13 +28,18 @@ import gettext
 import requests
 import re
 import signal
+import sys
+
+LANGDOMAIN = "slimbook"
+LANGDIR = "/usr/share/slimbook/locale-langpack"
 
 try:
     current_locale, encoding = locale.getdefaultlocale()
-    language = gettext.translation(APP, LANGDIR, [current_locale])
+    language = gettext.translation(LANGDOMAIN, LANGDIR, [current_locale])
     language.install()
     _ = language.gettext
 except Exception as e:
+    print(e, file = sys.stderr)
     _ = str
 
 SLB_EVENT_QC71_SILENT_MODE_CHANGED = 0x00
@@ -63,7 +68,15 @@ SLB_EVENT_BALANCED_MODE = 0x0800
 SLB_EVENT_PERFORMANCE_MODE = 0x0900
 
 SLB_EVENT_AC_OFFLINE = 0x1000
-SLB_EVENT_AC_ONLINE = 0x1100
+SLB_EVENT_AC_ONLINE = 0x1001
+
+SLB_EVENT_QC71_INPUT_LOADED = 0x2000
+SLB_EVENT_QC71_INPUT_UNLOADED = 0x2001
+
+SLB_EVENT_UPOWER_POWER_EVENT = 0x4000
+SLB_EVENT_UPOWER_POWER_SAVER = 0x4001
+SLB_EVENT_UPOWER_BALANCED    = 0x4002
+SLB_EVENT_UPOWER_PERFORMANCE = 0x4003
 
 SLB_EVENT_DATA = {
     SLB_EVENT_QC71_SILENT_MODE_ON : [_("Silent Mode enabled"),"power-profile-power-saver-symbolic"],
@@ -106,27 +119,60 @@ POWER_PROFILE_POWER_SAVER = "power-saver"
 POWER_PROFILE_BALANCED = "balanced"
 POWER_PROFILE_PERFORMANCE = "performance"
 
+POWER_PROFILE_NAME = {
+    slimbook.info.SLB_QC71_PROFILE_UNKNOWN : "unknown",
+    slimbook.info.SLB_QC71_PROFILE_ENERGY_SAVER : POWER_PROFILE_POWER_SAVER,
+    slimbook.info.SLB_QC71_PROFILE_BALANCED : POWER_PROFILE_BALANCED,
+    slimbook.info.SLB_QC71_PROFILE_PERFORMANCE : POWER_PROFILE_PERFORMANCE
+}
+
 TUNED_PROFILE = {
     POWER_PROFILE_POWER_SAVER : "powersave",
     POWER_PROFILE_BALANCED : "balanced-battery",
     POWER_PROFILE_PERFORMANCE : "throughput-performance"
     }
+    
+POWER_NAME_TO_EVENT = {
+    POWER_PROFILE_POWER_SAVER : SLB_EVENT_UPOWER_POWER_SAVER,
+    POWER_PROFILE_BALANCED : SLB_EVENT_UPOWER_BALANCED,
+    POWER_PROFILE_PERFORMANCE : SLB_EVENT_UPOWER_PERFORMANCE,
+    
+    #tuned names
+    TUNED_PROFILE[POWER_PROFILE_POWER_SAVER] : SLB_EVENT_UPOWER_POWER_SAVER,
+    TUNED_PROFILE[POWER_PROFILE_BALANCED] : SLB_EVENT_UPOWER_BALANCED,
+    TUNED_PROFILE[POWER_PROFILE_PERFORMANCE] : SLB_EVENT_UPOWER_PERFORMANCE
+}
 
 OPT_TRACKPAD_LOCK = "trackpad-lock"
 OPT_POWER_PROFILE = "power-profile"
 
 CMD_LOAD_SETTINGS = "cmd-load"
 
+QC71_DOUBLE_PROFILE = [slimbook.info.SLB_MODEL_PROX, slimbook.info.SLB_MODEL_EXECUTIVE]
+QC71_TRIPLE_PROFILE = [slimbook.info.SLB_MODEL_TITAN, slimbook.info.SLB_MODEL_HERO, slimbook.info.SLB_MODEL_EVO, slimbook.info.SLB_MODEL_CREATIVE]
+
+QC71_DOUBLE_PROFILE_FROM_UPOWER = {
+    SLB_EVENT_UPOWER_POWER_SAVER : slimbook.info.SLB_QC71_PROFILE_ENERGY_SAVER,
+    SLB_EVENT_UPOWER_BALANCED : slimbook.info.SLB_QC71_PROFILE_BALANCED,
+    SLB_EVENT_UPOWER_PERFORMANCE : slimbook.info.SLB_QC71_PROFILE_BALANCED
+}
+
+QC71_TRIPLE_PROFILE_FROM_UPOWER = {
+    SLB_EVENT_UPOWER_POWER_SAVER : slimbook.info.SLB_QC71_PROFILE_ENERGY_SAVER,
+    SLB_EVENT_UPOWER_BALANCED : slimbook.info.SLB_QC71_PROFILE_BALANCED,
+    SLB_EVENT_UPOWER_PERFORMANCE : slimbook.info.SLB_QC71_PROFILE_PERFORMANCE
+}
+
 #set a default dark theme for kde
 xdg_current_desktop = os.environ.get("XDG_CURRENT_DESKTOP")
 if xdg_current_desktop == "KDE":
     PARAMS['theme'] = 'dark'
 
 APP = 'slimbook'
-VERSION = '0.8.2'
+VERSION = '1.0'
 APPCONF = APP + '.conf'
 APPDATA = APP + '.data'
-APPNAME = 'Slimbook Service'
+APPNAME = _('Slimbook Service')
 CONFIG_DIR = os.path.join(os.path.expanduser('~'), '.config')
 CONFIG_APP_DIR = os.path.join(CONFIG_DIR, APP)
 CONFIG_FILE = os.path.join(CONFIG_APP_DIR, APPCONF)
